/*
 *
 *  ALL03
 *
 *  26.07.2002
 *
 *  by Fabio Sturman
 *
 *  fabio.sturman@tiscali.it
 *
 *  90slv.c
 *
 *  main module for programming AT90S8515, AT90S/LS2323 and
 *  AT90S/LS2343 in serial mode low voltage
 *
 */

/* Copyright (c) 2002 Fabio Sturman (fabio.sturman@tiscali.it)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>
#include "../include/all03.h"
#include "../include/all03ft.h"
#include "../include/keys.h"
#include "../include/sleep.h"
#include "../include/binmask.h"
#include "90slv.h"

/*- AT90SLV Level 1 --------------------------------------*/

int pin_miso=PIN_MISO_8515;
int pin_mosi=PIN_MOSI_8515;
int pin_sck=PIN_SCK_8515;
int pin_nreset=PIN_NRESET_8515;
int pin_vcc=PIN_VCC_8515;
int pin_clock=PIN_CLOCK_8515;

/*- AT90SLV Level 2 --------------------------------------*/

char chip_model[64]="AT90S8515";
char adapter_model[64]="A40OSC";
unsigned short int max_flash_address=0x1000;
unsigned short int max_eeprom_address=0x200;

unsigned short int flash[0x4000]; /* 32k */
unsigned char eeprom[0x400];      /* 1k  */

int lock_bit_1=1;
int lock_bit_2=1;
int spien=1;
int fstrt=1;
int rcen=1;

unsigned char signature[3];

char flash_file_name[256]= "flash90.bin";
char eeprom_file_name[256]="eeprom90.bin";


/*- AT90SLV Level 1 --------------------------------------*/

/* write read */
unsigned char wr_rd_serial(unsigned char byt)
{
  int i;
  unsigned char rd;

  rd=0;
  for(i=0; i<8; i++)
  {
    set_mosi(byt & BIT(7-i) ? 1 : 0);
    set_sck(1);
    if(get_miso()) rd|=BIT(7-i);
    set_sck(0);
  }
  return(rd);
}

/* put chip in programming mode */
int chip_init(void)
{
  all03_set_vcc(5.0);
  all03_set_gnd_20();
  set_power();

  set_nreset(0);
  set_sck(0);

  set_miso(1);

  if(pin_clock) set_clock();

  wait(30.0);

  if(cmd_programming_enable())
  {
    all03_init_h();
    printf("\n\007* ALL03 Error - no functional device connected!\n");
    return(1);
  }
  return(0);
}

int cmd_programming_enable(void)
{
  unsigned char t, t2;
  int i;

  for(i=0;i<32;i++)
  {
    t= wr_rd_serial(CMD_PROGRAMMING_ENABLE_1);
    t= wr_rd_serial(CMD_PROGRAMMING_ENABLE_2);
    t2=wr_rd_serial(0x00);
    t= wr_rd_serial(0x00);
    if(t2==CMD_PROGRAMMING_ENABLE_2) return(0);
    set_sck(1);
    set_sck(1);
    set_sck(0);
  }
  return(1);
}

void cmd_chip_erase(void)
{
  unsigned char t;

  t=wr_rd_serial(CMD_CHIP_ERASE_1);
  t=wr_rd_serial(CMD_CHIP_ERASE_2);
  t=wr_rd_serial(0x00);
  t=wr_rd_serial(0x00);
}

unsigned char cmd_read_program_memory(int lohi, unsigned short int address)
{
  unsigned char t;

  t=wr_rd_serial(CMD_READ_PROGRAM_MEMORY | (lohi ? BIT(3) : 0));
  t=wr_rd_serial((address >> 8) & M_0000_1111);
  t=wr_rd_serial(address & 0xff);
  t=wr_rd_serial(0x00);
  return(t);
}

void cmd_write_program_memory(int lohi, unsigned short int address, unsigned char data)
{
  unsigned char t;

  t=wr_rd_serial(CMD_WRITE_PROGRAM_MEMORY | (lohi ? BIT(3): 0));
  t=wr_rd_serial((address >> 8) & M_0000_1111);
  t=wr_rd_serial(address & 0xff);
  t=wr_rd_serial(data);
}

unsigned char cmd_read_eeprom_memory(unsigned short int address)
{
  unsigned char t;

  t=wr_rd_serial(CMD_READ_EEPROM_MEMORY);
  t=wr_rd_serial((address >> 8) & M_0000_0001);
  t=wr_rd_serial(address & 0xff);
  t=wr_rd_serial(0x00);
  return(t);
}

void cmd_write_eeprom_memory(unsigned short int address, unsigned char data)
{
  unsigned char t;

  t=wr_rd_serial(CMD_WRITE_EEPROM_MEMORY);
  t=wr_rd_serial((address >> 8) & M_0000_0001);
  t=wr_rd_serial(address & 0xff);
  t=wr_rd_serial(data);
}

/*  no 8515 */
unsigned char cmd_read_lock_and_fuse_bits(void)
{
  unsigned char t;

  t=wr_rd_serial(CMD_READ_LOCK_AND_FUSE_BITS);
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
  t=wr_rd_serial(0xff);
  return(t);
}

void cmd_write_lock_bits(int lock_bit_1, int lock_bit_2)
{
  unsigned char t;

  t=wr_rd_serial(CMD_WRITE_LOCK_BITS_1);
  t=wr_rd_serial(CMD_WRITE_LOCK_BITS_2 | (lock_bit_1 ? BIT(1) : 0) | (lock_bit_2 ? BIT(2) : 0) );
  t=wr_rd_serial(0x00);
  t=wr_rd_serial(0x00);
}

/* only 2323 */
void cmd_write_fstrt_bit(int nfstrt)
{
  unsigned char t;

  t=wr_rd_serial(CMD_WRITE_FSTRT_BIT_1);
  t=wr_rd_serial(CMD_WRITE_FSTRT_BIT_2 | (nfstrt ? BIT(0) : 0) );
  t=wr_rd_serial(0x00);
  t=wr_rd_serial(0x00);
}

unsigned char cmd_read_signature_bytes(unsigned short int address)
{
  unsigned char t;

  t=wr_rd_serial(CMD_READ_SIGNATURE_BYTES);
  t=wr_rd_serial(0x00);
  t=wr_rd_serial(0x00 | (address & M_0000_0011));
  t=wr_rd_serial(0x00);
  return(t);
}

int wait_rdy_program(int hilo, unsigned short int address, unsigned char data)
{
  long t, to;

  to=get_one_ms()*10;
  for(t=0; t<to; t++)
    if(data==cmd_read_program_memory(hilo,address)) return(0);
  return(1);
}

int wait_rdy_eeprom(unsigned short int address, unsigned char data)
{
  long t, to;

  to=get_one_ms()*10;
  for(t=0; t<to; t++)
    if(data==cmd_read_eeprom_memory(address)) return(0);
  return(1);
}

/*- AT90SLV Level 2 --------------------------------------*/

void select_model_2323(void)
{
  strcpy(chip_model,"AT90S/LS2323");
  strcpy(adapter_model,"A8T14VCC");
  max_flash_address =0x0400;
  max_eeprom_address=0x0080;
  pin_miso=PIN_MISO_2323;
  pin_mosi=PIN_MOSI_2323;
  pin_sck=PIN_SCK_2323;
  pin_nreset=PIN_NRESET_2323;
  pin_vcc=PIN_VCC_2323;
  pin_clock=PIN_CLOCK_2323;
}

void select_model_2343(void)
{
  strcpy(chip_model,"AT90S/LS2343");
  strcpy(adapter_model,"A8T14VCC");
  max_flash_address =0x0400;
  max_eeprom_address=0x0080;
  pin_miso=PIN_MISO_2323;
  pin_mosi=PIN_MOSI_2323;
  pin_sck=PIN_SCK_2323;
  pin_nreset=PIN_NRESET_2323;
  pin_vcc=PIN_VCC_2323;
  pin_clock=PIN_CLOCK_2323;
}

void select_model_8515(void)
{
  strcpy(chip_model,"AT90S81515");
  strcpy(adapter_model,"A40OSC");
  max_flash_address =0x1000;
  max_eeprom_address=0x0200;
  pin_miso=PIN_MISO_8515;
  pin_mosi=PIN_MOSI_8515;
  pin_sck=PIN_SCK_8515;
  pin_nreset=PIN_NRESET_8515;
  pin_vcc=PIN_VCC_8515;
  pin_clock=PIN_CLOCK_8515;
}

void edit_chip_model(void)
{
  int c;
  
  for(;;)
  {
    clear_screen();
    printf("*** EDIT CHIP MODEL ***\n\n");
    print_chip_model();
    printf("\n");
    printf("A) AT90S8515   B) AT90S/LS2323\n");
    printf("               D) AT90S/LS2343\n");
    printf("\n");
    printf("Q) QUIT\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<127)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);

    switch(c)
    {
      case 'A': select_model_8515(); break;
      case 'B': select_model_2323(); break;
      case 'D': select_model_2343(); break;

      case 'Q': return; break;
    }
  }
}

/* LOCK BITS AND FUSES ----------------------------------------------*/

/*
 *  disp_lock_and_fuse_bits()
 *
 *  display lock and fuse bits
 *
 */
void display_lock_and_fuse_bits(void)
{
  if(IS_AT90S_LS2323)
  {
    printf("Fuse and lock bits:\n\n");
    printf("SPIEN=      %d\n",spien ? 1 : 0);
    printf("FSTRT=      %d\n",fstrt ? 1 : 0);
  }
  else if(IS_AT90S_LS2343)
  {
    printf("Fuse and lock bits:\n\n");
    printf("SPIEN=      %d\n",spien ? 1 : 0);
    printf("RCEN=       %d\n",rcen  ? 1 : 0);
  }
  else if(IS_AT90S8515)
  {
    printf("Lock bits:\n\n");
  }
  printf("LOCK BIT 1= %d\n",lock_bit_1 ? 1 : 0);
  printf("LOCK BIT 2= %d\n",lock_bit_2 ? 1 : 0);

}

/*
 *  edit_lock_and_fuse_bits()
 *
 *  edit lock and fuse bits
 *
 */
void edit_lock_and_fuse_bits(void)
{
  int c;
  
  for(;;)
  {
    clear_screen();
    printf("*** EDIT LOCK BITS AND FUSES ***\n\n");
    display_lock_and_fuse_bits();
    printf("\n");
    if(IS_AT90S_LS2323)
    {
      printf("A) SPIEN <-     0     B) SPIEN <-     1\n");
      printf("C) FSTRT <-     0     D) FSTRT <-     1\n");
    }
    if(IS_AT90S_LS2343)
    {
      printf("A) SPIEN <-     0     B) SPIEN <-     1\n");
      printf("E) RCEN  <-     0     F) RCEN  <-     1\n");
    }
    printf("G) LOCK BIT1 <- 0     H) LOCK BIT1 <- 1\n");
    printf("I) LOCK BIT2 <- 0     J) LOCK BIT2 <- 1\n");
    printf("\n");
    printf("Q) QUIT\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<127)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);

    switch(c)
    {
      case 'A': spien=0; break;
      case 'B': spien=1; break;
      case 'C': fstrt=0; break;
      case 'D': fstrt=1; break;
      case 'E': rcen =0; break;
      case 'F': rcen= 1; break;
      case 'G': lock_bit_1=0; break;
      case 'H': lock_bit_1=1; break;
      case 'I': lock_bit_2=0; break;
      case 'J': lock_bit_2=1; break;
      case 'Q': return; break;
    }
  }
}

/*
 *  read_lock_and_fuse_bits()
 *
 *  read lock and fuse bits
 *  from chip
 *
 */
void read_lock_and_fuse_bits(void)
{
  unsigned char fuse;

  clear_screen();
  if(IS_AT90S8515)
  {
    print_no_8515();
    return;
  }
  if(test_if_timer()) return;
  printf("*** READ LOCK AND FUSE BITS ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");

  if(chip_init()) return;

  fuse=cmd_read_lock_and_fuse_bits();
  
  all03_init_h();

  lock_bit_1=0;
  lock_bit_2=0;
  spien=0;
  fstrt=0;
  rcen= 0;
  if(fuse & BIT(7)) lock_bit_1=1;
  if(fuse & BIT(6)) lock_bit_2=1;
  if(fuse & BIT(5)) spien=1;
  if(fuse & BIT(0)) fstrt=1;
  if(fuse & BIT(0)) rcen= 1;

  display_lock_and_fuse_bits();
  printf("OK\n");
}

/*
 *  write_fuse_bits()
 *
 *  write fuses
 *
 */
void write_fuse_bits(void)
{
  clear_screen();
  if(IS_AT90S8515)
  {
    print_no_8515();
    return;
  }
  if(test_if_timer()) return;
  printf("*** WRITE FUSE BITS ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  if(chip_init()) return;

  if(IS_AT90S_LS2323) cmd_write_fstrt_bit(fstrt);
  else if(IS_AT90S_LS2343) cmd_write_rcen_bit(rcen);

  all03_init_h();

  printf("OK\n");
}

/*
 *  write_lock()
 *
 *  write lock bits
 *
 */
void write_lock_bits(void)
{
  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE LOCK BITS ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");

  if(chip_init()) return;

  cmd_write_lock_bits(lock_bit_1, lock_bit_2);
  wait(20.0);

  all03_init_h();

  printf("OK\n");
}

/* ERASE -----------------------------------------------------------*/

/*
 *  chip_erase()
 *
 *  erase flash, eeprom and lock bits
 *  fuses unchanged
 *
 */
void chip_erase(void)
{
  clear_screen();
  if(test_if_timer()) return;
  printf("** ERASE ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("ERASING...\n");
  if(chip_init()) return;

  cmd_chip_erase();
  wait(20.0);

  all03_init_h();

  printf("OK\n");
}

/*
 *  read_signature_bytes()
 *
 *  reads signature bytes
 *
 *  90S8515
 *  SIG 0 - 0x00 manifactured by Atmel
 *  SIG 1 - 0x93 8k flash
 *  SIG 2 - 0x01 indicates AT90S8515 if SIG 1=0x93
 *
 *  90S/LS2323
 *  SIG 0 - 0x1E manifactured by Atmel
 *  SIG 1 - 0x91 2k flash
 *  SIG 2 - 0x02 indicates AT90S/LS2323 if SIG 1=0x91
 *
 *  90S2343
 *  SIG 0 - 0x1E manifactured by Atmel
 *  SIG 1 - 0x91 2k flash
 *  SIG 2 - 0x03 indicates AT90S/LS2343 if SIG 1=0x91
 *
 */
void read_signature_bytes(void)
{
  int i;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ SIGNATURE ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  if(chip_init()) return;
  

  for(i=0; i<3; i++)
  {
    signature[i]=cmd_read_signature_bytes(i);
  }

  all03_init_h();

  for(i=0;i<3;i++)
    printf("Signature[%d]=%02x\n", i, signature[i]);

  printf("OK\n");
}

/* FLASH - PROGRAM MEMORY  -----------------------------------*/

/*
 *  write_program_memory()
 *
 *  write flash buffer to uC flash memory
 *
 */
void write_program_memory(void)
{
  unsigned short int address;
  unsigned char d;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  if(chip_init()) return;


  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    if((d=(flash[address] & 0xff))!=0xff)
    {
      cmd_write_program_memory(LOW,address,d);
      if(d==0x7f) wait(10.0);
      else
      {
        if(wait_rdy_program(LOW,address,d))
        {
          printf("\n\007* ALL03 Error - Timeout in writing program memory at address %04x!\n",address);
          goto write_program_memory_q;
        }
      }
    }
    if((d=((flash[address]>>8) & 0xff))!=0xff)
    {
      cmd_write_program_memory(HIGH,address,d);
      if(d==0x7f) wait(10.0);
      else
      {
        if(wait_rdy_program(HIGH,address,d))
        {
          printf("\n\007* ALL03 Error - Timeout in writing program memory at address %04x!\n",address);
          goto write_program_memory_q;
        }
      }
    }
  }

write_program_memory_q:

  all03_init_h();
  printf("\n");

  printf("OK\n");
}

/*
 *  read_program_memory()
 *
 *  reads uC flash memory into flash buffer
 *
 */
void read_program_memory(void)
{
  unsigned short int address;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  if(chip_init()) return;

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    flash[address] =    cmd_read_program_memory(LOW, address); /* low data byte */
    flash[address]+=256*cmd_read_program_memory(HIGH,address); /* hi data byte */
  }

  all03_init_h();
  printf("\n");

  printf("OK\n");
}


/* EEPROM ------------------------------------------------*/

/*
 *  read_eeprom_memory()
 *
 *  reads uC eeprom memory into eeprom buffer
 *
 */
void read_eeprom_memory(void)
{

  unsigned short int address;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ EEPROM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  if(chip_init()) return;

  for(address=0; address<max_eeprom_address; address++)
  {
    printf("Address: %04x\r",address);
    eeprom[address]=cmd_read_eeprom_memory(address);
  }

  all03_init_h();
  printf("\n");

  printf("OK\n");

}

/*
 *  blank_check_eeprom()
 *
 *  verifies if all eeproms cells =0xff
 *
 */
void blank_check_eeprom_memory(void)
{

  int err;
  unsigned short int address;
  
  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** BLANK CHECK EEPROM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("CHECKING...\n");
  if(chip_init()) return;

  for(address=0; address<max_eeprom_address; address++)
  {
    printf("Address: %04x\r",address);
    if(cmd_read_eeprom_memory(address)!=0xff) err++;
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Eeprom memory not blank - %d errors!\n", err);
  else printf("Eeprom memory is blank\n");

  printf("OK\n");

}

/*
 *  verify_eeprom_memory()
 *
 *  verifies uC eeprom with eeprom buffer
 *
 */
void verify_eeprom_memory(void)
{
  int err;
  unsigned short int address;
  
  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** VERIFY EEPROM MEMORY ***\n\n");
  printf(PROC_STRING);
  wait_enter();

  printf("VERIFYING...\n");
  if(chip_init()) return;

  for(address=0; address<max_eeprom_address; address++)
  {
    printf("Address: %04x\r",address);
    if(eeprom[address]!=cmd_read_eeprom_memory(address)) err++;
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Counted %d errors!\n",err);
  else printf("No errors!");
  
  printf("OK\n");
}

/*
 *  write_eeprom_memory()
 *
 *  writes eeprom buffer into uC eeprom memory
 *
 */
void write_eeprom_memory(void)
{
  unsigned short int address;
  unsigned char d;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE EEPROM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  if(chip_init()) return;

  for(address=0; address<max_eeprom_address; address++)
  {
    printf("Address: %04x\r",address);
    if((d=eeprom[address])!=0xff)
    {
      cmd_write_eeprom_memory(address,d);
      if(d==0x7f) wait(10.0);
      else
      {
        if(wait_rdy_eeprom(address,d))
        {
          printf("\n\007* ALL03 Error - Timeout in writing eeprom memory at address %04x!\n",address);
          goto write_eeprom_memory_q;
        }
      }
    }
  }
  
write_eeprom_memory_q:
  
  all03_init_h();
  printf("\n");

  printf("OK\n");
}

/*
 *  blak_check()
 *
 *  verifies if all uC flash memory cells=0xff
 *
 */
void blank_check_program_memory(void)
{
  int err;
  unsigned short int address;

  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** BLANK CHECK PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("CHECKING...\n");
  if(chip_init()) return;

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    if(cmd_read_program_memory(LOW, address)!=0xff) err++; /* low data byte */
    if(cmd_read_program_memory(HIGH,address)!=0xff) err++; /* hi data byte */
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Flash not blank - %d errors!\n", err);
  else printf("Flash is blank!\n");

  printf("OK\n");
}

/*
 *  verify_program_memory()
 *
 *  verifies uC flash memory with flash buffer
 *
 */
void verify_program_memory(void)
{
  int err;
  unsigned short int address;

  err=0;
  clear_screen();
  if(test_if_timer()) return;
  printf("*** VERIFY PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("VERIFYING...\n");
  if(chip_init()) return;

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    if(( flash[address]    &0xff) != cmd_read_program_memory(LOW, address)) err++; /* low data byte */
    if(((flash[address]>>8)&0xff) != cmd_read_program_memory(HIGH,address)) err++; /* hi data byte */
  }

  all03_init_h();
  printf("\n");

  if(err) printf("Counted %d errors!\n",err);
  else printf("No errors!\n");
  
  printf("OK\n");
}

/*
 *  read_flash_buf_from_file()
 *
 *  reads data for flash buffer from disk
 *
 */
void read_program_memory_buffer_from_file(void)
{
  clear_screen();
  printf("*** READ FILE TO FLASH BUFFER ***\n\n");
  read_file_to_buf("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, max_flash_address*2, 0xff);
}


/*
 *  write_flash_buf_to_file()
 *
 *  writes data from flash buffer to disk
 *
 */
void write_program_memory_buffer_to_file(void)
{
  clear_screen();
  printf("*** WRITE FLASH BUFFER TO FILE ***\n\n");
  write_buf_to_file("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, max_flash_address*2);
}

/*
 *  read_eeprom_buf_from_file()
 *
 *  reads data for eeprom buffer from disk
 *
 */
void read_eeprom_memory_buffer_from_file(void)
{
  clear_screen();
  printf("*** READ FILE TO EEPROM BUFFER ***\n\n");
  read_file_to_buf("ENTER FILE NAME:  ",\
    eeprom_file_name, eeprom, max_eeprom_address, 0xff);
}

/*
 *  write_eeprom_buf_to_file()
 *
 *  writes data from eeprom buffer to disk
 *
 */
void write_eeprom_memory_buffer_to_file(void)
{
  clear_screen();
  printf("*** WRITE EEPROM BUFFER TO FILE ***\n\n");
  write_buf_to_file("ENTER FILE NAME:  ",\
    eeprom_file_name, eeprom, max_eeprom_address);
}

/* displays help screen */
void display_help(void)
{
    printf("use: 90slv [<option>] [<file_name>]\n");
    printf("option:\n");
    printf(" -s       skip processor speed test\n");
    printf(" -h       display this screen\n");
    printf(" -bxxx    set all03 base address\n");
    printf(" -t2323   select AT90S/LS2323\n");
    printf(" -t2343   select AT90S/LS2343\n");
    printf(" -t8515   select AT90S8515\n");

    exit(1);
}

/* main program */
int main(int argc, char *argv[])
{
  int c, ac, skip, tbase;

  all03_init_h();
  disable_ctrl_c_break();

  ac=argc-1;
  skip=0;
  while(ac)
  {
    if(!strcmp(argv[ac],"-s") || !strcmp(argv[ac],"-S")) skip=1;
    else if(!strcmp(argv[ac],"-h") || !strcmp(argv[ac],"-H"))
    {
      display_help();
      return(1);
    }
    else if(!strncmp(argv[ac],"-b",2) || !strncmp(argv[ac],"-B",2))
    {
      sscanf(&argv[ac][2],"%x",&tbase);
      all03_set_base_address(tbase);
    }
    else if(!strcmp(argv[ac],"-t2323") || !strcmp(argv[ac],"-T2323")) select_model_2323();
    else if(!strcmp(argv[ac],"-t2343") || !strcmp(argv[ac],"-T2343")) select_model_2343();
    else if(!strcmp(argv[ac],"-t8515") || !strcmp(argv[ac],"-T8515")) select_model_8515();
    else strcpy(flash_file_name,argv[ac]);
    ac--;
  }  

  if(!skip)
  {
    printf("TESTING SPEED...\n");
    if(!test_timer()) printf("TIMER OK\n");
    else printf("\n\007* ALL03 Error - Timing not OK!\n");
  }

  if(all03_test_base_address())
  {
    printf("\n\007* ALL03 Error - Programmer not found at base address %04x!\n",all03_get_base_address());
    printf("Press <ENTER>\n");
    wait_enter();
  }

  for(;;)
  {
    clear_screen();
    printf("-----------------------------------------------------------------\n");
    printf("-- ALL03 - AT90SLV SERIAL LOW VOLTAGE PROGRAMMING MODULE --------\n");
    printf("----------------------------------by FaSt - Ver 1.0BETA3---------\n");
    printf("\n");
    print_chip_model();
    printf("\n");
    printf("    Menu:\n");
    printf("\n");
    printf("2 - Read program file         4 - Read eeprom file\n");
    printf("3 - Save program file         5 - Save eeprom file\n");
    printf("6 - Edit base address\n");
    printf("B - Blank check               Z - Blank check eeprom\n");
    printf("R - Read                      O - Read eeprom\n");
    printf("P - Program                   F - Program eeprom\n");
    printf("L - Read fuse & lock bits\n");
    printf("I - Write lock bits           X - Write fuse bits\n");
    printf("S - Read signature\n");
    printf("E - Erase\n");
    printf("K - Edit fuse and lock bits\n");
    printf("V - Verify                    U - Verify eeprom\n");
    printf("T - Select chip model\n");
    //printf("{ - Test\n");
    //printf("} - Dump ALL03 registers\n");
    printf("Q - quit\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<=126)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);
    
    switch(c)
    {
      case '2': read_program_memory_buffer_from_file(); break;
      case '3': write_program_memory_buffer_to_file(); break;
      case '4': read_eeprom_memory_buffer_from_file(); break;
      case '5': write_eeprom_memory_buffer_to_file(); break;
      case '6': all03_edit_base(); break;

      case 'B': blank_check_program_memory(); break;
      case 'Z': blank_check_eeprom_memory(); break;
      case 'R': read_program_memory(); break;
      case 'O': read_eeprom_memory(); break;
      case 'S': read_signature_bytes(); break;
      case 'L': read_lock_and_fuse_bits(); break;
      case 'P': write_program_memory(); break;
      case 'F': write_eeprom_memory(); break;
      case 'I': write_lock_bits(); break;
      case 'X': write_fuse_bits(); break;
      case 'E': chip_erase(); break;
      case 'K': edit_lock_and_fuse_bits(); break;
      case 'V': verify_program_memory(); break;
      case 'U': verify_eeprom_memory(); break;
      case 'T': edit_chip_model(); break;

      case '{': test(); break;
      case '}': all03_dump(); break;

      case 'Q':
        enable_ctrl_c_break();
        return(0);
      break;
    }
    
    printf("Press <ENTER>\n");
    wait_enter();
  }
  return(0);
}

