/*
 *
 *  ALL03
 *
 *  14.05.2003
 *
 *  by Fabio Sturman
 *
 *  fabio.sturman@tiscali.it
 *
 *  pic16f84.c
 *
 *  main module for PIC16C[F]84[A]
 *
 */

/* Copyright (c) 2002, 2003 Fabio Sturman (fabio.sturman@tiscali.it)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#include <string.h>
#include "../include/all03.h"
#include "../include/all03ft.h"
#include "../include/keys.h"
#include "../include/sleep.h"
#include "../include/binmask.h"
#include "pic16x8y.h"

/*- PIC16C84 Level 1 --------------------------------------*/

int pin_sio=PIN_SIO_PIC16F84;
int pin_sck=PIN_SCK_PIC16F84;
int pin_nmclr=PIN_NMCLR_PIC16F84;
int pin_vcc=PIN_VCC_PIC16F84;

/*- PIC16C84 Level 2 --------------------------------------*/

char chip_model[64];
char adapter_model[64];
unsigned short int max_flash_address;
unsigned short int max_eprom_address;

unsigned short int flash[0x8000]; /* 32k */
unsigned char eprom[0x400];       /* 1k  */

/* configuration bits */
int cp=1;
int pwrite=1;
int wdte=0;
int fosc1=1;
int fosc0=0;
unsigned short int id[4]={0xf,0xf,0xf,0xf};
unsigned short int chip_id;

char flash_file_name[256]="led.bin"; // "flash.bin";
char eprom_file_name[256]="eprom.bin";

/*- PIC16C84 Level 1 --------------------------------------*/

/* write 6 bit command LSB first */
int wr_command(unsigned char byt)
{
  int i;

  set_sck(0);
  set_sio(0);

  for(i=0; i<6; i++)
  {
    set_sio(byt & BIT(i) ? 1 : 0);
    set_sck(1);
    set_sck(0);
  }
  wait(0.001);
  set_sio(1);
  wait(0.001);

  return(0);
}


/* write 14 bit data LSB first + traling and leading zero*/
int wr_data(unsigned short int dat)
{
  int i;

  // trailig 0
  set_sio(0);
  set_sck(1);
  set_sck(0);

  for(i=0; i<14; i++)
  {
    set_sio(dat & BIT(i) ? 1 : 0);
    set_sck(1);
    set_sck(0);
  }
  // leading zero
  set_sio(0);
  set_sck(1);
  set_sck(0);

  return(0);
}

/* read 14 bit data from chip LSB first */
unsigned short int rd_data(void)
{
  int i;
  unsigned short int td;

  td=0;
  // trailig 0
  set_sio(1);
  set_sck(0);

  set_sck(1);
  set_sck(0);

  for(i=0; i<14; i++)
  {
    set_sck(1);
    set_sck(0);
    if(get_sio()) td|= BIT(i);
  }
  // leading zero
  set_sck(1);
  set_sck(0);

  return(td);
}

#define LOW 0
#define HIGH 1
#define VIHH 2

void set_nmclr(int level)
{
  switch(level)
  {
    case LOW:
      all03_wr_vhh_pin(pin_nmclr,0);
      all03_wr_pin(pin_nmclr,0);
    break;

    case HIGH:
      all03_wr_vhh_pin(pin_nmclr,0);
      all03_wr_pin(pin_nmclr,1);
    break;
    case VIHH:
      all03_wr_pin(pin_nmclr,1);
      all03_wr_vhh_pin(pin_nmclr,1);
    break;
  }
}

/* put chip in programming mode */
int chip_init(void)
{
  all03_set_vcc(5.0);
  all03_set_vhh(13.0);
  all03_set_gnd_20();

  set_nmclr(0);
  set_sck(0);
  set_sio(0);

  set_power();
  wait(10.0);
  set_nmclr(VIHH);
  wait(10.0);
}


/*- PIC16F84 Level 2 --------------------------------------*/

void select_model_pic16f84(void)
{
  strcpy(chip_model,"PIC16F84");
  strcpy(adapter_model,"PIC16F84");
  max_flash_address =0x0400;
  max_eprom_address=0x0040;
  pin_sio=PIN_SIO_PIC16F84;
  pin_sck=PIN_SCK_PIC16F84;
  pin_nmclr=PIN_NMCLR_PIC16F84;
  pin_vcc=PIN_VCC_PIC16F84;
}

void select_model_pic16f83(void)
{
  strcpy(chip_model,"PIC16F83");
  strcpy(adapter_model,"PIC16F84");
  max_flash_address =0x0200;
  max_eprom_address=0x0040;
  pin_sio=PIN_SIO_PIC16F84;
  pin_sck=PIN_SCK_PIC16F84;
  pin_nmclr=PIN_NMCLR_PIC16F84;
  pin_vcc=PIN_VCC_PIC16F84;
}


int get_hex(char *s)
{
  int c;

  printf("\n[Enter HEX Digit] %s",s);
  for(;;)
  {
    c=key_get();
    if(isxdigit(c))
    {
      putchar(c);
      if(islower(c)) c=toupper(c);
      if(isdigit(c)) return c-0x30;
      else return c-0x41+10;
    }
    if(c==K_ESC) return -1;
  }
}


void edit_chip_model(void)
{
  int c;
  
  for(;;)
  {
    clear_screen();
    printf("*** EDIT CHIP MODEL ***\n\n");
    print_chip_model();
    printf("\n");
    printf("A) PIC16F83    B) PIC16F84\n");
    //printf("C) ********    D) ********\n");
    printf("\n");
    printf("Q) QUIT\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<127)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);

    switch(c)
    {
      case 'A': select_model_pic16f83(); break;
      case 'B': select_model_pic16f84(); break;

      case 'Q': return; break;
    }
  }
}

/* LOCK BITS AND FUSES ----------------------------------------------*/

/*
 *  display_configuration_bits()
 *
 *  display configuration bits
 *
 */
void display_configuration_bits(void)
{
  if(IS_PIC16F84 || IS_PIC16F83)
  {
    printf("Configuration bits:\n");
    printf("CP (Code protection ON=0):.......... %d\n",cp    ? 1 : 0);
    printf("PWRITE (Power up timer enable ON=0): %d\n",pwrite? 1 : 0);
    printf("WDTE (Watch dog enable ON=1):....... %d\n",wdte  ? 1 : 0);
    printf("FOSC1:.............................. %d\n",fosc1 ? 1 : 0);
    printf("FOSC0:.............................. %d\n",fosc0 ? 1 : 0);
    printf("  11=RC  10=HS  01=XT  00=LP\n");
    printf("ID[3..0]:........................... %1X%1X%1X%1X\n",\
      id[3]&0xf,id[2]&0xf,id[1]&0xf,id[0]&0xf);
    printf("CHIP ID:............................ %04X\n",chip_id);
  }
}


/*
 *  edit_configuration_bits()
 *
 *  edit configuration bits
 *
 */
void edit_configuration_bits(void)
{
  int c, rc;
  
  for(;;)
  {
    clear_screen();
    printf("*** EDIT CONFIGURATION BITS & ID ***\n\n");
    display_configuration_bits();
    printf("\n");
    if(IS_PIC16F84 || IS_PIC16F83)
    {
      printf("A) CP     <- 0     B) CP     <- 1\n");
      printf("C) PWRITE <- 0     D) PWRITE <- 1\n");
      printf("E) WDTE   <- 0     F) WDTE   <- 1\n");
      printf("G) FOSC1  <- 0     H) FOSC1  <- 1\n");
      printf("I) FOSC0  <- 0     J) FOSC0  <- 1\n");
      printf("K) Edit ID[3]      L) Edit ID[2]\n");
      printf("M) Edit ID[1]      N) Edit ID[0]\n");

    }
    printf("\n");
    printf("Q) QUIT\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<127)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);

    switch(c)
    {
      case 'A': cp=0; break;
      case 'B': cp=1; break;
      case 'C': pwrite=0; break;
      case 'D': pwrite=1; break;
      case 'E': wdte=0; break;
      case 'F': wdte=1; break;
      case 'G': fosc1=0; break;
      case 'H': fosc1=1; break;
      case 'I': fosc0=0; break;
      case 'J': fosc0=1; break;
      case 'K': if((rc=get_hex("ID[3]="))!=-1) id[3]=rc; break;
      case 'L': if((rc=get_hex("ID[2]="))!=-1) id[2]=rc; break;
      case 'M': if((rc=get_hex("ID[1]="))!=-1) id[1]=rc; break;
      case 'N': if((rc=get_hex("ID[0]="))!=-1) id[0]=rc; break;

      case 'Q': return; break;
    }
  }
}


/*
 *  read_configuration_bits()
 *
 *  read configuratione bits
 *  from chip
 *
 */
void read_configuration_bits(void)
{
  unsigned short int fuses, i;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ CONFIGURATION BITS & ID ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");

  chip_init();

  wr_command(CMD_LOAD_CONFIGURATION);
  wr_data(0x3fff);

  for(i=0;i<4;i++)
  {
    wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
    id[i]=rd_data();
    wr_command(CMD_INCREMENT_ADDRESS);
  }
  //4
  wr_command(CMD_INCREMENT_ADDRESS);
  //5
  wr_command(CMD_INCREMENT_ADDRESS);
  //6
  wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
  chip_id=rd_data();
  wr_command(CMD_INCREMENT_ADDRESS);
  //7
  wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
  fuses=rd_data();

  all03_init_h();

  cp=0;
  pwrite=0;
  wdte=0;
  fosc1=0;
  fosc0=0;
  if(fuses & BIT(4)) cp=1;
  if(fuses & BIT(3)) pwrite=1;
  if(fuses & BIT(2)) wdte=1;
  if(fuses & BIT(1)) fosc1=1;
  if(fuses & BIT(0)) fosc0=1;

  clear_screen();
  display_configuration_bits();
  printf("OK\n");
}


/*
 *  write_configuration_bits()
 *
 *  write configuration bits
 *
 */
void write_configuration_bits(void)
{
  unsigned short int fuses, rfuses, tid[4], i;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** WRITE CONFIGURATION BITS & ID ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("WRITING...\n");
  chip_init();

  fuses=0;
  if(cp==1) fuses|=(BIT(13)|BIT(12)|BIT(11)|BIT(10)| \
    BIT(9)|BIT(8)|BIT(7)|BIT(6)|BIT(5)|BIT(4));
  if(pwrite==1) fuses|=BIT(3);
  if(wdte==1) fuses|=BIT(2);
  if(fosc1==1) fuses|=BIT(1);
  if(fosc0==1) fuses|=BIT(0);

  wr_command(CMD_LOAD_CONFIGURATION); //0
  wr_data(id[0]);

  for(i=0;i<4;i++)
  {
    wr_command(CMD_LOAD_DATA_FOR_PROGRAM_MEMORY);
    wr_data(id[i]);
    wr_command(CMD_BEGIN_PROGRAMMING);
    wait(10.0);
    wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
    tid[i]=rd_data();
    wr_command(CMD_INCREMENT_ADDRESS);  //++
  }

  wr_command(CMD_INCREMENT_ADDRESS);  //5
  wr_command(CMD_INCREMENT_ADDRESS);  //6

  wr_command(CMD_INCREMENT_ADDRESS);  //7
  wr_command(CMD_LOAD_DATA_FOR_PROGRAM_MEMORY);
  wr_data(fuses);
  wr_command(CMD_BEGIN_PROGRAMMING);
  wait(10.0);
  wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
  rfuses=rd_data();
  if(rfuses!=fuses || tid[0]!=id[0] || tid[1]!=id[1] || tid[2]!=id[2] || tid[3]!=id[3])
  {
    printf("Error in writing configuration bits & id!\n");

//    printf("fuses: %04X  read: %04X\n",fuses,rfuses);
//    printf("id[0]: %1X   read: %1X\n",id[0],tid[0]);
//    printf("id[1]: %1X   read: %1X\n",id[1],tid[1]);
//    printf("id[2]: %1X   read: %1X\n",id[2],tid[2]);
//    printf("id[3]: %1X   read: %1X\n",id[3],tid[3]);
  }
  all03_init_h();

  printf("OK\n");
}


void chip_erase(void)
{
  unsigned short int fuses, rfuses;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** ERASING CHIP ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("ERASING...\n");
  chip_init();

  fuses=0x3fff; //a;

  wr_command(CMD_LOAD_CONFIGURATION);
  wr_data(fuses);

  wr_command(CMD_INCREMENT_ADDRESS);
  wr_command(CMD_INCREMENT_ADDRESS);
  wr_command(CMD_INCREMENT_ADDRESS);
  wr_command(CMD_INCREMENT_ADDRESS);
  wr_command(CMD_INCREMENT_ADDRESS);
  wr_command(CMD_INCREMENT_ADDRESS);
  wr_command(CMD_INCREMENT_ADDRESS);

  wr_command(1);
  wr_command(7);
  wr_command(CMD_BEGIN_PROGRAMMING);
  wait(10.0);
  wr_command(1);
  wr_command(7);

  all03_init_h();

  printf("OK\n");
}


/*
 *  write_program_memory()
 *
 *  write flash buffer to uC flash memory
 *
 */
void write_program_memory(int verify)
{
  unsigned short int address, data, errlo, errhi;

  clear_screen();
  if(test_if_timer()) return;
  if(!verify) printf("*** WRITE PROGRAM MEMORY ***\n");
  else printf("*** VERIFYING PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  if(!verify) printf("PROGRAMMING...\n");
  else printf("VERIFYING...\n");

  chip_init();

  errlo=0;
  errhi=0;

  if(!verify)
  {
    for(address=0; address<max_flash_address; address++)
    {
      printf("Address: %04x\r",address);
      wr_command(CMD_LOAD_DATA_FOR_PROGRAM_MEMORY);
      wr_data(flash[address]);
      wr_command(CMD_BEGIN_PROGRAMMING);
      wait(10.0);
      wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
      data=rd_data();
      if((0x3fff & flash[address])!=data)
      {
        printf("Error programming data %04X at address %04X\n",flash[address]&0x3fff,address);
        goto write_program_memory_q;
      }
      wr_command(CMD_INCREMENT_ADDRESS);
    }
  }

  printf("VERIFYING @4.5V...\n");
  all03_init_h();
  wait(10.0);
  chip_init();
  all03_set_vcc(4.5);
  wait(10.0);

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
    data=rd_data();
    if(data!=(flash[address] & 0x3fff)) errlo++;
    wr_command(CMD_INCREMENT_ADDRESS);
  }

  printf("VERIFYING @5.5V...\n");
  all03_init_h();
  wait(10.0);
  chip_init();
  all03_set_vcc(5.5);
  wait(10.0);

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
    data=rd_data();
    if(data !=(flash[address] & 0x3fff)) errhi++;
    wr_command(CMD_INCREMENT_ADDRESS);
  }

  if(errlo) printf("Counted %d errors @4.5V!\n",errlo);
  else printf("No errors @4.5V!\n");
  if(errhi) printf("Counted %d errors @5.5V!\n",errhi);
  else printf("No errors @5.5V!\n");

write_program_memory_q:

  all03_init_h();
  printf("\n");
  printf("OK\n");
}


/*
 *  read_program_memory()
 *
 *  reads uC flash memory into flash buffer
 *
 */
void read_program_memory(void)
{
  unsigned short int address, dat;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ PROGRAM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();

  for(address=0; address<max_flash_address; address++)
  {
    printf("Address: %04x\r",address);
    wr_command(CMD_READ_DATA_FROM_PROGRAM_MEMORY);
    flash[address]=rd_data();
    wr_command(CMD_INCREMENT_ADDRESS);
  }

  all03_init_h();
  printf("\n");
  printf("OK\n");
}


/*
 *  read_eprom_memory()
 *
 *  reads uC eeprom memory into eprom buffer
 *
 */
void read_eprom_memory(void)
{
  unsigned short int address;

  clear_screen();
  if(test_if_timer()) return;
  printf("*** READ EPROM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  printf("READING...\n");
  chip_init();

  for(address=0; address<max_eprom_address; address++)
  {
    printf("Address: %04x\r",address);
    wr_command(CMD_READ_DATA_FROM_DATA_MEMORY);
    eprom[address]=(unsigned char)rd_data();
    wr_command(CMD_INCREMENT_ADDRESS);
  }

  all03_init_h();
  printf("\n");

  printf("OK\n");
}



/*
 *  write_eprom_memory()
 *
 */
void write_eprom_memory(int verify)
{
  unsigned short int address, errlo, errhi;
  unsigned char data;

  clear_screen();
  if(test_if_timer()) return;
  if(!verify) printf("*** WRITE EPROM MEMORY ***\n");
  else printf("*** VERIFYING EPROM MEMORY ***\n");
  printf(PROC_STRING);
  wait_enter();

  if(!verify) printf("PROGRAMMING...\n");
  else printf("VERIFYING...\n");
  chip_init();

  errlo=0;
  errhi=0;
  if(!verify)
  {
    for(address=0; address<max_eprom_address; address++)
    {
      printf("Address: %04x\r",address);
      wr_command(CMD_LOAD_DATA_FOR_DATA_MEMORY);
      wr_data(eprom[address]);
      wr_command(CMD_BEGIN_PROGRAMMING);
      wait(10.0);
      wr_command(CMD_READ_DATA_FROM_DATA_MEMORY);
      data=(unsigned char)rd_data();
      if(eprom[address]!=data)
      {
        printf("Error programming data %02X at address %04X\n",eprom[address],address);
        goto write_eprom_memory_q;
      }
      wr_command(CMD_INCREMENT_ADDRESS);
    }
  }


  printf("VERIFYING @4.5V...\n");
  all03_init_h();
  wait(10.0);
  chip_init();
  all03_set_vcc(4.5);
  wait(10.0);

  for(address=0; address<max_eprom_address; address++)
  {
    printf("Address: %04x\r",address);
    wr_command(CMD_READ_DATA_FROM_DATA_MEMORY);
    data=(unsigned char)rd_data();
    if(data!=eprom[address]) errlo++;
    wr_command(CMD_INCREMENT_ADDRESS);
  }

  printf("VERIFYING @5.5V...\n");
  all03_init_h();
  wait(10.0);
  chip_init();
  all03_set_vcc(5.5);
  wait(10.0);

  for(address=0; address<max_eprom_address; address++)
  {
    printf("Address: %04x\r",address);
    wr_command(CMD_READ_DATA_FROM_DATA_MEMORY);
    data=(unsigned char)rd_data();
    if(data !=eprom[address]) errhi++;
    wr_command(CMD_INCREMENT_ADDRESS);
  }

  if(errlo) printf("Counted %d errors @4.5V!\n",errlo);
  else printf("No errors @4.5V!\n");
  if(errhi) printf("Counted %d errors @5.5V!\n",errhi);
  else printf("No errors @5.5V!\n");

write_eprom_memory_q:

  all03_init_h();
  printf("\n");
  printf("OK\n");
}



/*
 *  read_flash_buf_from_file()
 *
 *  reads data for flash buffer from disk
 *
 */
void read_program_memory_buffer_from_file(void)
{
  clear_screen();
  printf("*** READ FILE TO PROGRAM BUFFER ***\n\n");
  read_file_to_buf("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, max_flash_address*2, 0xff);
}


/*
 *  write_flash_buf_to_file()
 *
 *  writes data from flash buffer to disk
 *
 */
void write_program_memory_buffer_to_file(void)
{
  clear_screen();
  printf("*** WRITE PROGRAM BUFFER TO FILE ***\n\n");
  write_buf_to_file("ENTER FILE NAME:  ",\
    flash_file_name, (unsigned char *)flash, max_flash_address*2);
}


/*
 *  read_eprom_buf_from_file()
 *
 *  reads data for eprom buffer from disk
 *
 */
void read_eprom_memory_buffer_from_file(void)
{
  clear_screen();
  printf("*** READ FILE TO EPROM BUFFER ***\n\n");
  read_file_to_buf("ENTER FILE NAME:  ",\
    eprom_file_name, eprom, max_eprom_address, 0xff);
}


/*
 *  write_eprom_buf_to_file()
 *
 *  writes data from eprom buffer to disk
 *
 */
void write_eprom_memory_buffer_to_file(void)
{
  clear_screen();
  printf("*** WRITE EPROM BUFFER TO FILE ***\n\n");
  write_buf_to_file("ENTER FILE NAME:  ",\
    eprom_file_name, eprom, max_eprom_address);
}


/* displays help screen */
void display_help(void)
{
    printf("use: pic16f8x [<option>] [<file_name>]\n");
    printf("option:\n");
    printf(" -s       skip processor speed test\n");
    printf(" -h       display this screen\n");
    printf(" -bxxx    set all03 base address\n");
    printf(" -t16f84a select PIC16f84a\n");

    exit(1);
}

/* main program */
int main(int argc, char *argv[])
{
  int c, ac, skip, tbase;

  all03_init_h();
  disable_ctrl_c_break();

  ac=argc-1;
  skip=0;
  select_model_pic16f84();
  while(ac)
  {
    if(!strcmp(argv[ac],"-s") || !strcmp(argv[ac],"-S")) skip=1;
    else if(!strcmp(argv[ac],"-h") || !strcmp(argv[ac],"-H"))
    {
      display_help();
      return(1);
    }
    else if(!strncmp(argv[ac],"-b",2) || !strncmp(argv[ac],"-B",2))
    {
      sscanf(&argv[ac][2],"%x",&tbase);
      all03_set_base_address(tbase);
    }
    else if(!strcmp(argv[ac],"-t16f84") || !strcmp(argv[ac],"-T16F84")) select_model_pic16f84();
    else if(!strcmp(argv[ac],"-t16f83") || !strcmp(argv[ac],"-T16F83")) select_model_pic16f83();
    else strcpy(flash_file_name,argv[ac]);
    ac--;
  }  

  if(!skip)
  {
    printf("TESTING SPEED...\n");
    if(!test_timer()) printf("TIMER OK\n");
    else printf("\n\007* ALL03 Error - Timing not OK!\n");
  }

  if(all03_test_base_address())
  {
    printf("\n\007* ALL03 Error - Programmer not found at base address %04x!\n",all03_get_base_address());
    printf("Press <ENTER>\n");
    wait_enter();
  }

  for(;;)
  {
    clear_screen();
    printf("-----------------------------------------------------------------\n");
    printf("-- ALL03 - PIC16x8y PROGRAMMING MODULE --------------------------\n");
    printf("----------------------------------by FaSt - Ver 1.0ALPHA---------\n");
    printf("\n");
    print_chip_model();
    printf("\n");
    printf("    Menu:\n");
    printf("\n");
    printf("2 - Read program file         4 - Read eprom file\n");
    printf("3 - Save program file         5 - Save eprom file\n");
    printf("6 - Edit base address\n");
    printf("R - Read                      O - Read eprom\n");
    printf("P - Program                   F - Program eprom\n");
    printf("L - Read configuration bits\n");
    printf("I - Write configuration bits\n");
    printf("K - Edit configuration bits\n");
    printf("V - Verify                    U - Verify eprom\n");
    printf("T - Select chip model         E - Chip erase\n");
    //printf("{ - Test\n");
    //printf("} - Dump ALL03 registers\n");
    printf("Q - quit\n");
    printf("\n");
    printf("Enter command: ");
    flush_out();

    c=key_get();
    if((c>=32) && (c<=126)) printf("%c\n",c);
    if(islower(c)) c=toupper(c);
    
    switch(c)
    {
      case '2': read_program_memory_buffer_from_file(); break;
      case '3': write_program_memory_buffer_to_file(); break;
      case '4': read_eprom_memory_buffer_from_file(); break;
      case '5': write_eprom_memory_buffer_to_file(); break;
      case '6': all03_edit_base(); break;

      case 'R': read_program_memory(); break;
      case 'O': read_eprom_memory(); break;
      case 'L': read_configuration_bits(); break;
      case 'P': write_program_memory(0); break;
      case 'F': write_eprom_memory(0); break;
      case 'I': write_configuration_bits(); break;
      case 'K': edit_configuration_bits(); break;
      case 'V': write_program_memory(1); break;
      case 'U': write_eprom_memory(1); break;
      case 'T': edit_chip_model(); break;
      case 'E': chip_erase(); break;


      case '{': test(); break;
      case '}': all03_dump(); break;

      case 'Q':
        enable_ctrl_c_break();
        return(0);
      break;
    }
    
    printf("Press <ENTER>");
    wait_enter();
  }
  return(0);
}
